@@ -62,6 +62,12 @@ XWindowInterface::XWindowInterface(QObject *parent)
             , static_cast<void (KWindowSystem::*)(WId, NET::Properties, NET::Properties2)>
             (&KWindowSystem::windowChanged)
             , this, &XWindowInterface::windowChangedProxy);
+
+
+    for(auto wid : KWindowSystem::self()->windows()) {
+        emit windowAdded(wid);
+        windowChangedProxy(wid,0,0);
+    }
 }
 
 XWindowInterface::~XWindowInterface()
@@ -92,41 +98,41 @@ void XWindowInterface::setViewStruts(QWindow &view, const QRect &rect
     const QRect wholeScreen {{0, 0}, screen->virtualSize()};
 
     switch (location) {
-        case Plasma::Types::TopEdge: {
-            const int topOffset {screen->geometry().top()};
-            strut.top_width = rect.height() + topOffset;
-            strut.top_start = rect.x();
-            strut.top_end = rect.x() + rect.width() - 1;
-            break;
-        }
+    case Plasma::Types::TopEdge: {
+        const int topOffset {screen->geometry().top()};
+        strut.top_width = rect.height() + topOffset;
+        strut.top_start = rect.x();
+        strut.top_end = rect.x() + rect.width() - 1;
+        break;
+    }
 
-        case Plasma::Types::BottomEdge: {
-            const int bottomOffset {wholeScreen.bottom() - currentScreen.bottom()};
-            strut.bottom_width = rect.height() + bottomOffset;
-            strut.bottom_start = rect.x();
-            strut.bottom_end = rect.x() + rect.width() - 1;
-            break;
-        }
+    case Plasma::Types::BottomEdge: {
+        const int bottomOffset {wholeScreen.bottom() - currentScreen.bottom()};
+        strut.bottom_width = rect.height() + bottomOffset;
+        strut.bottom_start = rect.x();
+        strut.bottom_end = rect.x() + rect.width() - 1;
+        break;
+    }
 
-        case Plasma::Types::LeftEdge: {
-            const int leftOffset = {screen->geometry().left()};
-            strut.left_width = rect.width() + leftOffset;
-            strut.left_start = rect.y();
-            strut.left_end = rect.y() + rect.height() - 1;
-            break;
-        }
+    case Plasma::Types::LeftEdge: {
+        const int leftOffset = {screen->geometry().left()};
+        strut.left_width = rect.width() + leftOffset;
+        strut.left_start = rect.y();
+        strut.left_end = rect.y() + rect.height() - 1;
+        break;
+    }
 
-        case Plasma::Types::RightEdge: {
-            const int rightOffset = {wholeScreen.right() - currentScreen.right()};
-            strut.right_width = rect.width() + rightOffset;
-            strut.right_start = rect.y();
-            strut.right_end = rect.y() + rect.height() - 1;
-            break;
-        }
+    case Plasma::Types::RightEdge: {
+        const int rightOffset = {wholeScreen.right() - currentScreen.right()};
+        strut.right_width = rect.width() + rightOffset;
+        strut.right_start = rect.y();
+        strut.right_end = rect.y() + rect.height() - 1;
+        break;
+    }
 
-        default:
-            qWarning() << "wrong location:" << qEnumToStr(location);
-            return;
+    default:
+        qWarning() << "wrong location:" << qEnumToStr(location);
+        return;
     }
 
     KWindowSystem::setExtendedStrut(view.winId(),
@@ -134,7 +140,7 @@ void XWindowInterface::setViewStruts(QWindow &view, const QRect &rect
                                     strut.right_width,  strut.right_start,  strut.right_end,
                                     strut.top_width,    strut.top_start,    strut.top_end,
                                     strut.bottom_width, strut.bottom_start, strut.bottom_end
-                                   );
+                                    );
 }
 
 void XWindowInterface::switchToNextVirtualDesktop() const
@@ -206,24 +212,24 @@ void XWindowInterface::slideWindow(QWindow &view, AbstractWindowInterface::Slide
     auto slideLocation = KWindowEffects::NoEdge;
 
     switch (location) {
-        case Slide::Top:
-            slideLocation = KWindowEffects::TopEdge;
-            break;
+    case Slide::Top:
+        slideLocation = KWindowEffects::TopEdge;
+        break;
 
-        case Slide::Bottom:
-            slideLocation = KWindowEffects::BottomEdge;
-            break;
+    case Slide::Bottom:
+        slideLocation = KWindowEffects::BottomEdge;
+        break;
 
-        case Slide::Left:
-            slideLocation = KWindowEffects::LeftEdge;
-            break;
+    case Slide::Left:
+        slideLocation = KWindowEffects::LeftEdge;
+        break;
 
-        case Slide::Right:
-            slideLocation = KWindowEffects::RightEdge;
-            break;
+    case Slide::Right:
+        slideLocation = KWindowEffects::RightEdge;
+        break;
 
-        default:
-            break;
+    default:
+        break;
     }
 
     KWindowEffects::slideWindow(view.winId(), slideLocation, -1);
@@ -264,26 +270,26 @@ void XWindowInterface::setEdgeStateFor(QWindow *view, bool active) const
     uint32_t value = 0;
 
     switch (window->location()) {
-        case Plasma::Types::TopEdge:
-            value = 0;
-            break;
+    case Plasma::Types::TopEdge:
+        value = 0;
+        break;
 
-        case Plasma::Types::RightEdge:
-            value = 1;
-            break;
+    case Plasma::Types::RightEdge:
+        value = 1;
+        break;
 
-        case Plasma::Types::BottomEdge:
-            value = 2;
-            break;
+    case Plasma::Types::BottomEdge:
+        value = 2;
+        break;
 
-        case Plasma::Types::LeftEdge:
-            value = 3;
-            break;
+    case Plasma::Types::LeftEdge:
+        value = 3;
+        break;
 
-        case Plasma::Types::Floating:
-        default:
-            value = 4;
-            break;
+    case Plasma::Types::Floating:
+    default:
+        value = 4;
+        break;
     }
 
     int hideType = 0;
@@ -301,21 +307,21 @@ WindowInfoWrap XWindowInterface::requestInfoActive() const
 WindowInfoWrap XWindowInterface::requestInfo(WindowId wid) const
 {
     const KWindowInfo winfo{wid.value<WId>(), NET::WMFrameExtents
-                            | NET::WMWindowType
-                            | NET::WMGeometry
-                            | NET::WMDesktop
-                            | NET::WMState
-                            | NET::WMName
-                            | NET::WMVisibleName,
-                              NET::WM2WindowClass
-                            | NET::WM2Activities
-                            | NET::WM2TransientFor};
+                | NET::WMWindowType
+                | NET::WMGeometry
+                | NET::WMDesktop
+                | NET::WMState
+                | NET::WMName
+                | NET::WMVisibleName,
+                NET::WM2WindowClass
+                | NET::WM2Activities
+                | NET::WM2TransientFor};
 
     //! update desktop id
 
-    bool isPlasmaDesktop{false};
-    if (winfo.windowClassName() == "plasmashell" && hasScreenGeometry(winfo)) {
-        isPlasmaDesktop = true;
+    bool isDesktop{false};
+    if (winfo.windowClassName() == "plasmashell" && isPlasmaDesktop(winfo.geometry())) {
+        isDesktop = true;
         windowsTracker()->setPlasmaDesktop(wid);
     }
 
@@ -323,7 +329,7 @@ WindowInfoWrap XWindowInterface::requestInfo(WindowId wid) const
 
     if (!winfo.valid()) {
         winfoWrap.setIsValid(false);
-    } else if (isValidWindow(winfo) && !isPlasmaDesktop) {
+    } else if (isValidWindow(winfo) && !isDesktop) {
         winfoWrap.setIsValid(true);
         winfoWrap.setWid(wid);
         winfoWrap.setParentId(winfo.transientFor());
@@ -389,8 +395,8 @@ QUrl XWindowInterface::windowUrl(WindowId wid) const
     }
 
     return windowUrlFromMetadata(info.windowClassClass(),
-        NETWinInfo(QX11Info::connection(), wid.value<WId>(), QX11Info::appRootWindow(), NET::WMPid, NET::Properties2()).pid(),
-        rulesConfig, info.windowClassName());
+                                 NETWinInfo(QX11Info::connection(), wid.value<WId>(), QX11Info::appRootWindow(), NET::WMPid, NET::Properties2()).pid(),
+                                 rulesConfig, info.windowClassName());
 }
 
 
@@ -403,6 +409,7 @@ bool XWindowInterface::windowCanBeDragged(WindowId wid) const
         return (winfo.isValid()
                 && info.actionSupported(NET::ActionMove)
                 && !winfo.isMinimized()
+                && inCurrentDesktopActivity(winfo)
                 && !winfo.isPlasmaDesktop());
     }
 
@@ -418,6 +425,7 @@ bool XWindowInterface::windowCanBeMaximized(WindowId wid) const
         return (winfo.isValid()
                 && !winfo.isMinimized()
                 && info.actionSupported(NET::ActionMax)
+                && inCurrentDesktopActivity(winfo)
                 && !winfo.isPlasmaDesktop());
     }
 
@@ -462,11 +470,11 @@ void XWindowInterface::requestMoveWindow(WindowId wid, QPoint from) const
 {
     WindowInfoWrap wInfo = requestInfo(wid);
 
-    if (!wInfo.isValid() || wInfo.isPlasmaDesktop()) {
+    if (!wInfo.isValid() || wInfo.isPlasmaDesktop() || !inCurrentDesktopActivity(wInfo)) {
         return;
     }
 
-    int borderX{wInfo.geometry().width() > 120 ? 60 : 10};
+    int borderX = wInfo.geometry().width() > 120 ? 60 : 10;
     int borderY{10};
 
     //! find min/max values for x,y based on active window geometry
@@ -525,7 +533,7 @@ void XWindowInterface::requestToggleMinimized(WindowId wid) const
 {
     WindowInfoWrap wInfo = requestInfo(wid);
 
-    if (!wInfo.isValid() || wInfo.isPlasmaDesktop()) {
+    if (!wInfo.isValid() || wInfo.isPlasmaDesktop() || !inCurrentDesktopActivity(wInfo)) {
         return;
     }
 
@@ -544,11 +552,12 @@ void XWindowInterface::requestToggleMinimized(WindowId wid) const
 
 void XWindowInterface::requestToggleMaximized(WindowId wid) const
 {
-    if (!windowCanBeMaximized(wid)) {
+    WindowInfoWrap wInfo = requestInfo(wid);
+
+    if (!windowCanBeMaximized(wid) || !inCurrentDesktopActivity(wInfo)) {
         return;
     }
 
-    WindowInfoWrap wInfo = requestInfo(wid);
     bool restore = wInfo.isMaxHoriz() && wInfo.isMaxVert();
 
     if (wInfo.isMinimized()) {
@@ -616,20 +625,6 @@ bool XWindowInterface::isValidWindow(const KWindowInfo &winfo) const
     return !(isMenu || isDock);
 }
 
-bool XWindowInterface::hasScreenGeometry(const KWindowInfo &winfo) const
-{
-    bool hasScreenGeometry{false};
-
-    for (const auto scr : qGuiApp->screens()) {
-        if (!winfo.geometry().isEmpty() && winfo.geometry() == scr->geometry()) {
-            hasScreenGeometry = true;
-            break;
-        }
-    }
-
-    return hasScreenGeometry;
-}
-
 void XWindowInterface::windowChangedProxy(WId wid, NET::Properties prop1, NET::Properties2 prop2)
 {
     const KWindowInfo info(wid, NET::WMGeometry, NET::WM2WindowClass);
@@ -641,12 +636,17 @@ void XWindowInterface::windowChangedProxy(WId wid, NET::Properties prop1, NET::P
         return;
     }
 
-    //! update desktop id
-    if (winClass == "plasmashell" && hasScreenGeometry(info)) {
-        m_desktopId = wid;
-        windowsTracker()->setPlasmaDesktop(wid);
-        considerWindowChanged(wid);
-        return;
+    if (winClass == "plasmashell") {
+        //! update desktop id
+        if (isPlasmaDesktop(info.geometry())) {
+            m_desktopId = wid;
+            windowsTracker()->setPlasmaDesktop(wid);
+            considerWindowChanged(wid);
+            return;
+        } else if (isPlasmaPanel(info.geometry())) {
+            registerPlasmaPanel(wid);
+            return;
+        }
     }
 
     //! accept only NET::Properties events,
@@ -660,12 +660,12 @@ void XWindowInterface::windowChangedProxy(WId wid, NET::Properties prop1, NET::P
     //! accept only the following NET:Properties changed signals
     //! NET::WMState, NET::WMGeometry, NET::ActiveWindow
     if ( !(prop1 & NET::WMState)
-          && !(prop1 & NET::WMGeometry)
-          && !(prop1 & NET::ActiveWindow)
-          && !(prop1 & NET::WMDesktop)
-          && !(prop1 & (NET::WMName | NET::WMVisibleName)
-          && !(prop2 & NET::WM2TransientFor)
-          && !(prop2 & NET::WM2Activities)) ) {
+         && !(prop1 & NET::WMGeometry)
+         && !(prop1 & NET::ActiveWindow)
+         && !(prop1 & NET::WMDesktop)
+         && !(prop1 & (NET::WMName | NET::WMVisibleName)
+              && !(prop2 & NET::WM2TransientFor)
+              && !(prop2 & NET::WM2Activities)) ) {
         return;
     }
 
